#include <stdio.h>

#include "sofi.h"

#include "../common/modes.h"

/* The list of arguments should be:					*/
/*	-sallfacts -snomr -snohr -snomin *.o files			*/
int main (argc, argv)
    int argc;
    char **argv;
{
    char	*files [1024];
    int		numFiles = 0;
    FILE	*f, *m;
    char	fileName [1024], line [1024];
    int         i, j, numElements, pos, loc, c;
    char	itemName [1024];
    Item	*ptr, *next, **base;

    /* Go through options */
    for (i = 1 ; i < argc ; i++)
    {
	if (strcmp (argv [i], "-sallfacts"))
	    allFactsOption = 1;
	else if (strcmp (argv [i], "-snomr"))
	    noMrOption = 1;
	else if (strcmp (argv [i], "-snohr"))
	    noHrOption = 1;
	else if (strcmp (argv [i], "-snomin"))
	    noMinOption = 1;
	else
	{
	    int len = strlen (argv [i]);
	    if ((len > 2) && 
		(argv [i][len - 2] == '.') && (argv [i][len - 1] == 'o'))
	    {
		files [numFiles++] = argv [i];
	    }
	}
    }

    if (numFiles == 0)
    {
	fprintf (stderr, "No files to sofi link!\n", fileName);
	exit (201);
    }

    /* Read the information from each file, if possible */
    for (i = 0 ; i < numFiles ; i++)
    {
	CreateFileName (file [i], fileName, 0);
	fileName [strlen (fileName) - 1] = "c";

	/* Open the .sofi file produced by cpp */
	f = fopen (SOFI_INFO_FILE, RMODE);
	if (f == NULL)
	{
	    fprintf (stderr, "%s could not be found!\n", fileName);
	    exit (101);
	}

	/*******************/
	/* Read CC1 header */
	/*******************/
	ReadAndVerifyHeader (f, fileName, CC1_HEADER);

	/* Read the file list into the file table */
	ReadFileListIntoTable (f);

	fileTable [1].comparisonValue = 
		    SofiAddFile (fileTable [1].fileName, 
				 fileTable [1].shortFileName, -2, -2);
	for (i = 2 ; i <= numFiles ; i++)
	{
	    fileTable [i].comparisonValue = 
			SofiAddFile (fileTable [i].fileName, 
				     fileTable [i].shortFileName, -1, -1);
	}
	
	/* Read Include Data */
	mainHead = NULL;
	ReadMRDataIntoDataLL (f);

	ptr = mainHead;
	while (ptr != NULL)
	{
	    if (ptr->fileAttributed >= 0)
	    {
		ptr->fileAttributed = 
				fileTable [ptr->fileAttributed].comparisonValue;
	    }
	    if (ptr->subjectFile >= 0)
	    {
		ptr->subjectFile = fileTable [ptr->subjectFile].comparisonValue;
	    }
	    if (ptr->objectFile >= 0)
	    {
		ptr->objectFile = fileTable [ptr->objectFile].comparisonValue;
	    }
	    next = ptr->next;
	    ptr->next = masterHead;
	    masterHead = ptr;
	    ptr = next;
	}

	free (fileTable);

	fclose(f);
    }

    /* Add in the "External-Library" file */
    sofiExternalFileNumber = SofiAddFile ("External-Library", 
					  "External-Library", -1, -1);

    /* Convert the list of files to a table */
    ConvertFileLLToTable ();

    /* We now have a huge linked list, with file numbers corresponding	*/
    /* to the file link list.						*/

    /* Now we should sort up link imports with the link exports and 	*/
    /* assign values to the objectFile and objectLine fields.		*/
    CorrelateLinkRefsAndDefs (masterHead);

    /* Now we've done that, let's turn the data into a sorted table	*/
    base = ConvertDataLLToTableAndSort (masterHead, 	/* Sort List */
				        &numElements, CompareItems);

    /* Let's write them out */
    if (!noMrOption)
    {
	loc = 0;

	/* We'll send everything to one file for now */
	f = fopen (SOFI_MR_INFO_FILE, WMODE);		/* Open file */
	fprintf (f, FILE_HEADER, TimeString ());	/* Write header */
	WriteFileListFromFileTable (f);		/* Write file table */
	while (loc < numElements)
	{
	    loc = WriteMRDataFromDataTable (f, base, loc); 
						    /* Write MR data */
	}
	fclose (f);					/* Close file */
    }

    loc = 0;

    if (!noHrOption)
    {
	f = fopen (SOFI_HR_INFO_FILE, WMODE);		/* Open file */
	fprintf (f, FILE_HEADER, TimeString ());	/* Write header */
    }
    if (!noMinOption)
    {
	m = fopen (SOFI_MIN_INFO_FILE, AMODE);	/* Open File */
	fprintf (m, FILE_HEADER, TimeString ());	/* Write header */
    }
    while (loc < numElements)
    {
	loc = WriteHRDataFromDataTable (f, m, base, loc);
						    /* Write HR data */
    }
    if (!noMinOption)
    {
	fclose (m);					/* Close file */

    }
    if (!noHrOption)
    {
	fclose (f);					/* Close file */
    }
    free (base);					/* Free memory */
}
