#include <stdio.h>
#include <string.h>

#define LINE_SIZE	8192

#define IS_WHITE_SPACE(ch) ((ch == ' ') || (ch == '\t') || (ch == ':') || \
			    (ch == ';') || (ch == '&') || (ch == '\'') || \
			    (ch == '"') || (ch == '\'') || (ch == '=') || \
			    (ch == '$'))
#define IS_FILE_CHAR(ch) ((('a' <= ch) && (ch <= 'z')) || \
			  (('A' <= ch) && (ch <= 'Z')) || \
			  (('0' <= ch) && (ch <= '9')) || \
			  ((ch == '.') || (ch == '_') || (ch == '-')))
main (argc, argv)
    int argc;
    char *argv [];
{
    char	*inputFile, *outputFile;
    char	*linePtr, line [LINE_SIZE];
    char	*tokenPtr, token [LINE_SIZE];
    FILE	*in, *out;
    int		lineNum = 0, len;

    if ((argc != 2) && (argc != 3))
    {
	fprintf (stderr, 
	    "usage: cfx_make_trans <input make file> {<output make file>}\n");
	exit (1);
    }

    inputFile = argv [1];
    if (argc == 3)
    {
	outputFile = argv [2];
	if (strcmp (inputFile, outputFile) == 0)
	{
	    exit (3);
	}
    }
    else
    {
	outputFile = (char *) malloc (strlen (inputFile) + 10);
	if (outputFile == NULL)
	{
	    exit (2);
	}
	strcpy (outputFile, inputFile);
	strcat (outputFile, ".cfx");
    }

    in = fopen (inputFile, "r");
    if (in == NULL)
    {
	fprintf (stderr, "Unable to open \"%s\" for reading\n", inputFile);
	exit (4);
    }

    out = fopen (outputFile, "w");
    if (out == NULL)
    {
	fprintf (stderr, "Unable to open \"%s\" for writing\n", outputFile);
	exit (5);
    }

    /* Write out those changes that are made to all Makefiles */
    fprintf (out, "# Added by cfx_make_trans\n\n");
    fprintf (out, "# Added to autocreate .cfx files from .c files\n");
    fprintf (out, ".cfx.%%.o: %%.c\n");
    fprintf (out, "	$(COMPILE.c) $< $(OUTPUT_OPTION)\n\n");
    fprintf (out, "# Added to override cc, ld and ar\n");
    fprintf (out, "CC = cfx_cc\n");
    fprintf (out, "LD = cfx_ld\n");
    fprintf (out, "AR = cfx_ar\n");
    fprintf (out, "# End of additions made by cfx_make_trans\n\n");

    /* For every line in the file */
    for (;;)
    {
	if (fgets (line, LINE_SIZE, in) == NULL)
	{
	    if (feof (in)) break;
	    else
	    {
		fprintf (stderr, "Read error while reading \"%s\"\n", 
			 inputFile);
	    	exit (6);
	    }
	}

	lineNum++;

	len = strlen (line);

	if (len == LINE_SIZE - 1)
	{
	    fprintf (stderr, "Line %d too long in \"%s\"\n", lineNum, 
		     inputFile);
	    exit (6);
	}

	if (line [len - 1] == '\n')
	{
	    line [len - 1] = 0;
	}

	/* This loop reads white space then a token */
	linePtr = line;
	while (*linePtr != 0)
	{
	    /* Read in a send out white space */
	    while ((!IS_FILE_CHAR (*linePtr)) && (*linePtr != 0))
	    {
		fputc (*linePtr++, out);
	    }

	    if (*linePtr == 0) break;

	    tokenPtr = token;
	    while (IS_FILE_CHAR (*linePtr))
	    {
		*tokenPtr++ = *linePtr++;
	    }
	    *tokenPtr = 0;

	    /************************************************************/
	    /* We now have a single token.  We want to change:		*/
	    /*	   cc -> cfx_cc						*/
	    /*     ld -> cfx_ld						*/
	    /*	   ar -> cfx_ar						*/
	    /*     <file>.o to .cfx.<file>.o				*/
	    /************************************************************/
	    if (strcmp (token, "cc") == 0)
	    {
		fputs ("cfx_cc", out);
	    }
	    else if (strcmp (token, "ld") == 0)
	    {
		fputs ("cfx_ld", out);
	    }
	    else if (strcmp (token, "ar") == 0)
	    {
		fputs ("cfx_ar", out);
	    }
	    else
	    {
		len = strlen (token);
		if (((token [len - 2] == '.') && (token [len - 1] == 'o')) ||
		    ((token [len - 3] == '.') && (token [len - 2] == 'o') &&
		     (token [len - 1] == ':')))
		{
		    if (strchr (token, '/') == NULL)
		    {
			fputs (".cfx.", out);
			fputs (token, out);
		    }
		    else
		    {
			char	*ptr = strrchr (token, '/');

			*ptr = 0;
			fputs (token, out);
			fputs ("/.cfx.", out);
			ptr++;
			fputs (ptr, out);
		    }
		}
		else
		{
		    fputs (token, out);
		}
	    }
	}

	fputc ('\n', out);
    } /* for (;;) */

    fprintf (stderr, "%d lines translated\n", lineNum);

    fclose (in);
    fclose (out);
}
