#include "cfx-canon.h"

#define EOS '\0'

static void ChdirError(char *path)
{
    char str[512];

    sprintf(str, "Error chdir '%s'", path);

    perror(str);
}

/* HEY, fails on pathnames ending in "." or ".."! */

char *Cfx_CanonPath(char *fileName) {

  struct stat *statbuf;
  int linklen;
  int statret;
  int dirtmp;

  static char *original_path = NULL;
  char shortname[BUFSIZE];
  char linkexpan[BUFSIZE];
  char path[BUFSIZE];

  char *temprchr;

  T tempdata;
  Node *tempNode;

  /* FIRST check the cache to see if this fileName has already been
     resolved! */
  strcpy(tempdata.key, fileName);
  if (tempNode = findNode(tempdata)) {
    return tempNode->data.resolved;
  }

  statbuf = (struct stat *) malloc(sizeof(struct stat));

  if (!original_path) {
    original_path = (char *) malloc(BUFSIZE + 1);
    if (!getcwd(original_path, BUFSIZE)) {
      perror("getcwd error!");
    }
  }

  if (fileName[strlen(fileName) - 1] == '/') {
    fileName[strlen(fileName) - 1] = EOS;
  }

  strcpy(path, fileName);

/* Begin loop */

  while ( (linklen = readlink(path, linkexpan, BUFSIZE)) > -1) {

    linkexpan[linklen] = EOS;
    if ( (statret = stat(fileName, statbuf)) == -1) {
      perror("stat error!");
      printf("stat errno: %d\n", errno);
    }
    else {
      if ( (statbuf->st_mode & S_IFMT) == S_IFDIR) {
      }
      /* What we have is a symlink to another file
         Replace the symlink with its expansion, and try this whole
         thing again.
      */
      if (linkexpan[0] == (char) '/') {
        /*
         If the expansion starts with a "/", replace the entire path
         with it.
        */
        strcpy(path, linkexpan);
      } 
      else {
        /* Chop the last field off path and replace it with linkexpan */
        /* This will fail if the path ends with a "/"!!! We should get
           rid of those in the beginning.
              Then again, assuming we are always looking at a FILE not
              a directory, we be ok! */

        if ( (temprchr = (char *) strrchr(path, (char) '/')) == NULL) {
           strcpy(path, linkexpan);
        }
        else {
          path[strlen(path) - strlen(temprchr) + 1] = EOS;
          strcat(path, linkexpan);
        }
      }
    }
  }


  if (linklen != -1) {
    printf ("This should not have happened!\n");
  }
  else {
   /* perror("readlink:"); */
    if (errno == EINVAL) {
    }
    else {
      if (errno == ENOENT) {
        return fileName;
      }
      else {
        printf("A serious readlink error has occurred. narf.\n");
        exit(3);
      }
    }
  }

  /* Now we chop the fileName off path, chdir to the resulting directory,
     and get_cwd there. Sticking the fileName onto the result will give us
     the canonical name for that file! Then we chdir back. */ 

  if ( (temprchr = (char *) strrchr(path, (char) '/')) == NULL) {
    strcpy(shortname, path);
    /* get the cwd  - is path suitable for this? */
    if (!getcwd(path, BUFSIZE)) {
      perror("getcwd error!");
    }
    /* stick a slash onto the end of it */
    strcat(path, "/"); 
    /* stick the fileName onto the end of it */
    strcat(path, shortname); 
  }
  else {
    strcpy(shortname, temprchr);
    /* Chop off the fileName */
    path[strlen(path) - strlen(temprchr) + 1] = EOS;
    if ( (dirtmp = chdir(path)) == 0) {
      /* get the cwd  - is path suitable for this? */
      /* Path has been malloc()ed BUFSIZE */
      if (!getcwd(path, BUFSIZE)) {
        perror("getcwd error!");
      }
      /* stick the fileName onto the end of it */
      strcat(path, shortname); 
      /* Change directories back. */
      if ( (dirtmp = chdir(original_path)) != 0) {
	ChdirError(original_path);
      }
    }
    else {
      ChdirError(path);
    }
  }

  /* Add this to the cache */
  strcpy(tempdata.key, fileName);
  strcpy(tempdata.resolved, path);
  return (insertNode(tempdata))->data.resolved;
/*
  return path;
*/

}
