#include "cfx-hash.h"

/* Perhaps the "key" and "resolved" in "struct T" ought to be pointers -
   kinda weighty to be passing around this struct with two 2048-character
   strings between all these functions...
*/

/* *****************************
  Currently, will keep chaining until we run out of memory.
  Set a limit on how long a chain can get. If this is exceeded, rehash?
*/

hashTableIndex hash(T data) {

   char temp[BUFSIZE];
   char *temptr;
   int i;
   int sum = 0;
   /***********************************
    *  hash function applied to data  *
    ***********************************/
    /* return (data % hashTableSize); */ 

    /* Must return an int from 0 to hashTableSize 
       Going to take the substring following the last slash in the key
       (and excluding any slash at the end) and use the sum of the characters
       as the index.
    */

    strcpy(temp, data.key);
    if (data.key[strlen(data.key)-1] == (char) '/') {
      temp[strlen(temp)-1] = 0;
    }
    temptr = (char *) strrchr(temp, '/');
    if (!temptr) {
      temptr = &temp[0] - 1;
    }
    for (i = 1; i < strlen(temptr); i++) {
    /*  printf("temptr[%d]: %c\n", i, temptr[i]); */
      sum += (char) temptr[i];
    }
    /* printf("sum: %d\n", sum); */
    return (sum % hashTableSize);
}

Node *insertNode(T data) {
    Node *p, *p0;
    hashTableIndex bucket;
    T temp;

   /************************************************
    *  allocate node for data and insert in table  *
    ************************************************/

    /* insert node at beginning of list */
    bucket = hash(data);
    if ((p = malloc(sizeof(Node))) == 0) {
        fprintf (stderr, "out of memory (insertNode)\n");
        exit(1);
    }
    p0 = hashTable[bucket];
    hashTable[bucket] = p;
    p->next = p0;
    strcpy(p->data.key, data.key);
    strcpy(p->data.resolved, data.resolved);
    /* Now let's also insert an entry whose "key" is this entry's "resolved".
       This is because I know that cfx will later call Cfx_CanonPath with
       the current "resolved" as the fileName. */
    strcpy(temp.key, data.resolved);
    if (!findNode(temp))  {
      strcpy(temp.resolved, data.resolved);
      insertNode(temp);
    }
    return p;
}

void deleteNode(T data) {
    Node *p0, *p;
    hashTableIndex bucket;

   /********************************************
    *  delete node containing data from table  *
    ********************************************/

    /* find node */
    p0 = 0;
    bucket = hash(data);
    p = hashTable[bucket];
    while (p && !compEQ(p->data.key, data.key)) {
        p0 = p;
        p = p->next;
    }
    if (!p) return;

    /* p designates node to delete, remove it from list */
    if (p0)
        /* not first node, p0 points to previous node */
        p0->next = p->next;
    else
        /* first node on chain */
        hashTable[bucket] = p->next;

    free (p);
}

Node *findNode (T data) {
    Node *p;

   /*******************************
    *  find node containing data  *
    *******************************/

    p = hashTable[hash(data)];
    while (p && !compEQ(p->data.key, data.key)) 
        p = p->next;
    return p;
}

/*
void main() {
  T blah, foo, bar;
  strcpy(blah.key, "/char.c");
  strcpy(blah.resolved, "/h/5/brewste/char.c/");
  strcpy(bar.key, "oi.c");
  strcpy(bar.resolved, "/h/5/brewste/oi.c");

  insertNode(blah);
  insertNode(bar);

  strcpy(foo.key, "oi.c");
  printf("%s\n", findNode(foo)->data.resolved);

  strcpy(foo.key, "/h/5/brewste/char.c/");
  printf("%s\n", findNode(foo)->data.resolved);

} 
*/
